import ipaddress
import re

from django.core import validators
from django.db import models
from django.utils.translation import gettext_lazy as _


class SortableIPAddressField(models.CharField):
    description = _("IP address field (IPv4 or IPv6 with optional CIDR and VRF (max 63 Chars)")

    def __init__(self, *args, **kwargs):
        kwargs["max_length"] = 100  # Adjust as needed for longest possible representation
        super().__init__(*args, **kwargs)
        self.validators.append(self.validate_ip_address)

    def validate_ip_address(self, value):
        if not value:  # Allow blank values if not required
            return

        match = re.match(r"^(.+?)(?:/(\d+))?(?:@(.+))?$", value)
        if not match:
            raise validators.ValidationError(
                _("Invalid IP address format."),
                code="invalid",
            )

        ip_str = match.group(1)
        cidr = match.group(2)
        vrf = match.group(3)

        if vrf is not None:
            if len(vrf) > 63:
                raise validators.ValidationError(
                    _("VRF must be 63 characters or less."),
                    code="invalid",
                )

        try:
            ip_obj = ipaddress.ip_address(ip_str)
        except ValueError:
            raise validators.ValidationError(
                _("Invalid IP address."),
                code="invalid",
            )

        if cidr:
            try:
                cidr = int(cidr)
                if not (0 <= cidr <= (32 if ip_obj.version == 4 else 128)):
                    raise validators.ValidationError(
                        _("Invalid CIDR prefix length."),
                        code="invalid",
                    )
            except ValueError:
                raise validators.ValidationError(
                    _("Invalid CIDR prefix length."),
                    code="invalid",
                )
        elif ip_obj.version == 4:
            cidr = 32
        else:
            cidr = 128

    def to_python(self, value):
        if not value:
            return None

        match = re.match(r"^(.+?)(?:/(\d+))?(?:@(.+))?$", value)
        if not match:
            raise validators.ValidationError(
                _("Invalid IP address format."),
                code="invalid",
            )
        ip_str = match.group(1)
        cidr = match.group(2)
        vrf = match.group(3)

        try:
            ip_obj = ipaddress.ip_address(ip_str)
        except ValueError:
            raise validators.ValidationError(
                _("Invalid IP address."),
                code="invalid",
            )

        if cidr:
            cidr = int(cidr)
        elif ip_obj.version == 4:
            cidr = 32
        else:
            cidr = 128

        return f"{ip_str}/{cidr}@{vrf}" if vrf else f"{ip_str}/{cidr}" if cidr else ip_str

    def get_prep_value(self, value):
        if value is None:
            return None

        match = re.match(r"^(.+?)(?:/(\d+))?(?:@(.+))?$", value)
        if not match:
            raise ValueError("Invalid IP address format.")  # Should be caught by validation

        ip_str = match.group(1)
        cidr = match.group(2)
        vrf = match.group(3)

        try:
            ip_obj = ipaddress.ip_address(ip_str)
        except ValueError:
            raise ValueError("Invalid IP address.")

        ip_hex = hex(int(ip_obj)).replace("0x", "").zfill(32)  # Pad with zeros to 32 chars
        version = 4 if ip_obj.version == 4 else 6

        if cidr is None:
            cidr = 32 if ip_obj.version == 4 else 128
        cidr_hex = hex(int(cidr))[2:].zfill(2)

        vrf_str = vrf or ""

        return f"{version}{ip_hex}^{cidr_hex}@{vrf_str}"

    def from_db_value(self, value, expression, connection):
        if value is None:
            return None

        version = int(value[0])
        ip_hex = value[1:33]
        cidr_hex = value[34:36]
        vrf = value[37:]

        ip_int = int(ip_hex, 16)
        if version == 4:
            ip_str = str(ipaddress.IPv4Address(ip_int))
        else:
            ip_str = str(ipaddress.IPv6Address(ip_int))

        cidr = int(cidr_hex, 16)

        return (
            f"{ip_str}/{cidr}@{vrf}" if vrf else f"{ip_str}/{cidr}" if cidr != (32 if version == 4 else 128) else ip_str
        )

    def db_type(self, connection):
        return "char(100)"  # Or appropriate length for your use case
